#include <bits/stdc++.h>
using namespace std;
ifstream in("input.txt");
ofstream out("output.txt");
constexpr int INF = 1e9;

int n, m;
vector<vector<int>> graph;
vector<bool> is_main_cycle, onstack, visited;
vector<int> parent;
vector<int> main_cycle, main_cycle_pos;

vector<int> min_reach, max_reach;

void out_cycle(const vector<int>& cycle) {
    out << cycle.size();
    for (int u : cycle)
        out << " " << u;
    out << endl;
}

bool find_main_cycle_rec(int u, int to_ignore) {
    onstack[u] = true;
    for (int v : graph[u]) {
        if (v == to_ignore) {
            // ignoro
        } else if (onstack[v]) { // ho trovato un ciclo
            while (u != v) {
                main_cycle.push_back(u);
                u = parent[u];
            }
            main_cycle.push_back(u);
            return true;
        } else if (!visited[v]) {
            parent[v] = u;
            if (find_main_cycle_rec(v, to_ignore)) {
                return true;
            }
        }
    }
    onstack[u] = false;
    visited[u] = true;

    return false;
}

/**
 * Trova un ciclo senza il nodo to_ignore
 */
void find_main_cycle(int to_ignore = -1) {
    visited.assign(n, false);
    onstack.assign(n, false);
    parent.assign(n, -1);
    main_cycle.clear();
    bool cycle_found = false;
    for (int u = 0; u < n && !cycle_found; u++) {
        if (u != to_ignore && !visited[u]) {
            cycle_found = find_main_cycle_rec(u, to_ignore);
        }
    }

    is_main_cycle.assign(n, false);
    main_cycle_pos.assign(n, -1);
    reverse(main_cycle.begin(), main_cycle.end());

    for (int i = 0; i < main_cycle.size(); i++) {
        int u = main_cycle[i];
        main_cycle_pos[u] = i;
        is_main_cycle[u] = true;
    }
}

/*
 * Assegna min_reach[u] = nodo del ciclo principale più a sinistra raggiungibile da u
 * Assegna max_reach[u] = nodo del ciclo principale più a destra raggiungibile da u
 * Restituisce il nodo del ciclo principale più a destra raggiungibile da un salto all'indietro.
 */
int dfs_backward_loops(int u, int leave_pos) {
    visited[u] = true;
    int max_back = -1;
    for (int v : graph[u]) {
        if (is_main_cycle[v]) {
            int v_pos = main_cycle_pos[v];
            if (v_pos <= leave_pos) {
                max_back = max(max_back, v_pos);
            }
            min_reach[u] = min(min_reach[u], v_pos);
            max_reach[u] = max(max_reach[u], v_pos);
        } else {
            if (!visited[v]) {
                max_back = max(max_back, dfs_backward_loops(v, leave_pos));
            }
            min_reach[u] = min(min_reach[u], min_reach[v]);
            max_reach[u] = max(max_reach[u], max_reach[v]);
        }
    }
    return max_back;
}

/**
 * Trova un candidato nel ciclo principale non escluso da salti in avanti o indietro.
 */
int find_candidate_in_main_cycle() {
    // -------- Esclusione dei candidati per "salti all'indietro" -------
    visited.assign(n, false);
    min_reach.resize(n, INF);
    max_reach.resize(n, -1);

    int candidates_begin = 0, candidates_end = main_cycle.size();
    for (int s_pos = main_cycle.size() - 1; s_pos >= 0; s_pos--) {
        int s = main_cycle[s_pos];

        // escludo i nodi alla sinistra di un nodo raggiunto da un salto all'indietro
        candidates_begin = max(candidates_begin, dfs_backward_loops(s, s_pos));

        // escludo i nodi alla destra di un nodo da cui parte un salto all'indietro
        if (min_reach[s] <= s_pos) {
            candidates_end = s_pos + 1;
        }
    }

    if (candidates_begin >= candidates_end) {
        return -1;
    }

    // -------- Esclusione dei candidati per "salti in avanti" -------
    for (int s_pos = 0; s_pos < candidates_begin; s_pos++) {
        int s = main_cycle[s_pos];

        // salto in avanti da s a max_reach[s]
        if (max_reach[s] > s_pos) {
            candidates_begin = max(candidates_begin, max_reach[s]);
        }

        if (candidates_begin >= candidates_end) { // nessun candidato rimasto
            return -1;
        }
    }

    return candidates_begin;
}

void find_cycles_intersection() {
    find_main_cycle();
    assert(!main_cycle.empty());

    int candidate_pos = find_candidate_in_main_cycle();

    vector<int> first_cycle = main_cycle;

    int x;
    if (-1 == candidate_pos) {
        // cerco il secondo ciclo senza x
        x = first_cycle.front();
    } else {
        // Se x è un candidato, devo controllare se togliendolo elimino ogni ciclo.
        // Questo è necessario per controllare che un eventuale candidato sia davvero una soluzione,
        // in quanto potrebbero esserci cicli disgiunti da quello principale.
        x = main_cycle[candidate_pos];
    }

    find_main_cycle(x);

    if (main_cycle.empty()) {
        out << x << endl;
        out_cycle(first_cycle);
    } else {
        out << -1 << endl;
        out << x << endl;
        out_cycle(first_cycle);
        out_cycle(main_cycle);
    }
}

int main() {
    in >> n >> m;
    graph.resize(n);

    for (int i = 0; i < m; i++) {
        int u, v;
        in >> u >> v;
        graph[u].push_back(v);
    }

    find_cycles_intersection();
}
