/**
 * Per risolvere i casi più semplici possiamo assumere che tutti i cicli
 * siano disgiunti.
 * => i cicli corrispondono alle ogni componente fortemente connesse con
 *    almeno due nodi
 *
 * Troviamo le SCC con almeno due nodi:
 * - Se ce n'è una sola, allora esiste un solo ciclo e possiamo scegliere
 *   qualunque nodo di quel ciclo.
 * - Altrimenti, la risposta è impossibile.
 */
#include <bits/stdc++.h>

using namespace std;
#ifdef EVAL
ifstream in("input.txt");
ofstream out("output.txt");
#else
istream& in(cin);
ostream& out(cout);
#endif

int n, m;
vector<vector<int>>
    g,  // grafo
    gT; // grafo trasposto
vector<bool> visited;

void print_cycle(const vector<int>& cycle) {
    out << cycle.size();
    for (auto it = cycle.rbegin(); it != cycle.rend(); ++it)
        out << " " << *it;
    out << endl;
}

void topsort_rec(const int u, vector<int>& stack) {
    visited[u] = true;
    for (int v : g[u])
        if (!visited[v])
            topsort_rec(v, stack);
    stack.push_back(u);
}

vector<int> topsort() {
    vector<int> stack;
    visited.assign(n, false);
    for (int u = 0; u < n; u++)
        if (!visited[u])
            topsort_rec(u, stack);
    return stack;
}

void visit_scc(const int u, vector<int>& scc) {
    visited[u] = true;
    scc.push_back(u);
    for (int v : gT[u])
        if (!visited[v])
            visit_scc(v, scc);
}

vector<vector<int>> find_sccs() {
    vector<int> stack = topsort();
    vector<vector<int>> sccs;

    visited.assign(n, false);
    for (auto it = stack.rbegin(); it != stack.rend(); ++it) {
        if (!visited[*it]) {
            sccs.emplace_back();
            visit_scc(*it, sccs.back());
        }
    }

    return sccs;
}

int main() {
    in >> n >> m;
    g.resize(n);
    gT.resize(n);

    for (int i = 0; i < m; i++) {
        int u, v;
        in >> u >> v;
        g[u].push_back(v);
        gT[v].push_back(u);
    }

    vector<vector<int>> sccs = find_sccs();
    vector<int> cycle_1, cycle_2;
    for (const auto& scc : sccs) {
        if (scc.size() > 1) {
            if (cycle_1.empty())
                cycle_1 = scc;
            else {
                cycle_2 = scc;
                break;
            }
        }
    }
    assert(!cycle_1.empty());
    if (cycle_2.empty()) {
        out << cycle_1.front() << endl;
        print_cycle(cycle_1);
    } else {
        out << -1 << "\n"
            << cycle_1.front() << "\n";
        print_cycle(cycle_1);
        print_cycle(cycle_2);
    }
}
